import re
import os
from datetime import timedelta as tdelta
import numpy as np

def sl(sample, unit):
  x = sample//unit
  if x*unit < sample:
    return (x+1)*unit
  return sample

#returns array of [sample, messages]
def parsec(chat, sample_len, total_len, signal=False):
  msg = 0
  sample, begin = None, None
  datalist, sigs = [], []
  rgx = re.compile("^\[.*?\]")
  sig = re.compile("!twigen")
  total_len = tdelta(seconds=total_len)
  with open(chat, encoding='utf8') as f:
    for line in f:
      m = rgx.findall(line)
      if signal:
        s = sig.findall(line)
        if len(s) != 0:
          begin  = to_tdelta(m[0])
          sample = Sample(begin, sample_len)
          sigs.append(sample)
          continue
      if len(m) != 0:
        begin = to_tdelta(m[0])
        if begin == None:
          continue
        if sample == None:
          sample = Sample(begin, sample_len)
          if sample.end > total_len:
            sample.end = total_len
          msg = 1
        elif begin >= sample.end:
          datalist.append([sample,msg])
          if sample.end == total_len:
            break
          sample = None
        else:
          msg += 1
    if sample != None and begin < sample.end:
      datalist.append([sample,msg])
  return np.array(datalist), sigs

#returns normalized data
#min not needed, no negative values
def norm(data):
  d = data
  d[:,1] /= d[:,1].max()
  return d

#returns data whose elements in the second column are gte than limit
def cut(data, limit):
  d = data
  return d[np.where(d[:,1] >= limit)]

#func for numpy vectorization, delay samples in data
def delays(sample, delay):
  delay = tdelta(seconds=delay)
  zero  = tdelta()
  new   = Sample(sample.begin - delay,
                 length=sample.length.seconds + delay.seconds)
  if new.begin < zero:
    return sample
  else:
    return new

#apply delays to data
def do_delays(data, delays, delay):
  func = np.vectorize(delays, otypes=[Sample])
  return func(data[:,0], delay).tolist()

#return samples from data
def samples(data):
  d = data
  while True:
    merges = 0
    merged = False
    samples = []
    if len(d) < 2:
      samples.extend(d)
      return samples
    for i in range(1, len(d)):
      after  = d[i]
      before = d[i-1]
      if before.end >= after.begin:
        samples.append(Sample.merge(before, after))
        merged = True
        merges+= 1
      else:
        if merged:
          merged = False
          continue
        elif i == len(d)-1:
          samples.extend([before, after])
        else:
          samples.append(before)
    if merges == 0:
      break
    else:
     d = samples
  return samples

#read string or file and parse samples with format
#'00:00:00, 00:01:00[;|\n] 02:00:00, 03:00:00'
def includes(samples, inc, total_len, sigs=[]):
  if not inc:
    if not sigs:
      return samples
    else:
      return sorted((samples+sigs), key=lambda s: s.begin)
  incs = []
  if os.path.exists(inc):
    with open(inc) as f:
      for line in f:
        incs.append(line)
  else:
    incs = inc.split(';')
  t_len = tdelta(seconds=total_len)
  d     = samples
  res   = []
  for sample in incs:
    s = sample.split(',')
    s0 = to_tdelta(s[0])
    s1 = to_tdelta(s[1])
    if s0 > s1:
      s0, s1 = s1, s0
    if s1 > t_len:
      s1 = self.total_len
    res.append(Sample(s0, length=(s1-s0).total_seconds()))
  d.extend(res+sigs)
  d.sort(key=lambda s: s.begin)
  return d

class Sample:
  def __init__(self, begin = tdelta(), length = 0):
    self.length = tdelta(seconds=length)
    self.begin  = begin
    self.end    = begin+self.length

  @staticmethod
  def merge(s1, s2):
    return Sample(s1.begin, s2.end.total_seconds() - s1.begin.total_seconds())
  
  def __repr__(self):
    return "("+str(self.begin)+", "+str(self.end)+")"

def to_tdelta(string):
  rgx   = re.compile("[0-9]+")
  match = rgx.findall(string)
  if len(match) != 3:
    return None
  return tdelta(hours   = int(match[0]),
                minutes = int(match[1]),
                seconds = int(match[2]))
