import sys
import subprocess as sp
import tkinter as tk
import datetime as dt

class ArgBuilder():
  def __init__(self, arglist, win):
    self.arg  = arglist
    self.win  = win
    self.proc = None
  def runwith(self, *args):
    args_t = list(filter(lambda t: t[1] != '', args))
    args   = []
    for t in args_t:
      args.extend([t[0], t[1]])
    self.arg.extend([*args])
    print(' '.join(self.arg))
    self.proc = sp.Popen(self.arg)
    self.win.destroy()
    self.proc.wait()
    sys.exit()
  def setbool(self, argval, value):
    if value:
      if argval not in self.arg:
        self.arg.append(argval)
    else:
      self.arg = list(filter(lambda x: x != argval, self.arg))

def run():
  # set main window
  win = tk.Tk(className="twigen")
  winh, winw = 500, 800
  can = tk.Canvas(win, height=winh, width=winw)
  can.pack()

  twiargs = ArgBuilder(['twigen'], win)

  main_bg = '#0F0E11'

  # main frame
  main_f = tk.Frame(win, bg=main_bg)
  main_f.place(relwidth=1, relheight=1)

  # logo frame TODO use tk.PhotoImage
  logo_f = tk.Frame(main_f)
  logo_f.place(relx=0, rely=0, relwidth=1, relheight=0.2)
  logo_label = tk.Label(logo_f, text="twigen", font=(None, 40), bg=main_bg, fg='#FFFFFF')
  logo_label.place(relheight=1, relwidth=1)

  label_args = dict(font=(None, 14), bg=main_bg, fg='#FFFFFF')
  check_args = dict(font=(None, 14), bg=main_bg, fg='#FFFFFF', highlightbackground=main_bg, activebackground='#4B367C', activeforeground='#FFFFFF', selectcolor='#4B367C', width=50, anchor='w')
  entry_args = dict(font=14, width=60, bd=0, highlightcolor='#FFFFFF')
  entry_h    = .05
  sep        = .01

  # urls entry
  urls   = tk.StringVar()
  urls_y = .2 + sep
  urls_f = tk.Frame(main_f, bg=main_bg)
  urls_f.place(relx=.05, rely=urls_y, relwidth=1, relheight=entry_h)
  urls_label = tk.Label(urls_f, text="VODs", **label_args)
  urls_entry = tk.Entry(urls_f, **entry_args, textvariable=urls)
  urls_label.pack(side=tk.LEFT)
  urls_entry.place(relx=.12)

  # includes entry
  incs   = tk.StringVar()
  incs_y = urls_y + entry_h + sep
  incs_f = tk.Frame(main_f, bg=main_bg)
  incs_f.place(relx=.05, rely=incs_y, relwidth=1, relheight=entry_h)
  incs_label = tk.Label(incs_f, text="Include", **label_args)
  incs_entry = tk.Entry(incs_f, **entry_args, textvariable=incs)
  incs_label.pack(side=tk.LEFT)
  incs_entry.place(relx=.12)

  # output entry
  out   = tk.StringVar()
  out_y = incs_y + entry_h + sep
  out_f = tk.Frame(main_f, bg=main_bg)
  out_f.place(relx=.05, rely=out_y, relwidth=1, relheight=entry_h)
  out_label = tk.Label(out_f, text="Output", **label_args)
  out_entry = tk.Entry(out_f, **entry_args, textvariable=out)
  out_label.pack(side=tk.LEFT)
  out_entry.place(relx=.12)

  # min entry
  mins   = tk.StringVar()
  mins_y = out_y + entry_h + sep
  mins_f = tk.Frame(main_f, bg=main_bg)
  mins_f.place(relx=.05, rely=mins_y, relwidth=1, relheight=entry_h)
  mins_label = tk.Label(mins_f, text="Min Dur.", **label_args)
  mins_entry = tk.Entry(mins_f, **entry_args, textvariable=mins)
  mins_label.pack(side=tk.LEFT)
  mins_entry.place(relx=.12)

  # sample and delay lists TODO

  # bool args checkboxs TODO disable MEGs checkboxs
  group   = tk.BooleanVar(False)
  group_y = mins_y + entry_h + sep*2
  group_f = tk.Frame(main_f, bg=main_bg)
  group_f.place(relx=.05, rely=group_y, relwidth=1, relheight=entry_h)
  group_check = tk.Checkbutton(group_f, text="Group URLs into a single output file", variable=group, onvalue=True, offvalue=False, **check_args, command=lambda: twiargs.setbool('--group', group.get()))
  group_check.pack(side=tk.LEFT)

  segs   = tk.BooleanVar(False)
  segs_y = group_y + entry_h + sep
  segs_f = tk.Frame(main_f, bg=main_bg)
  segs_f.place(relx=.05, rely=segs_y, relwidth=1, relheight=entry_h)
  segs_check = tk.Checkbutton(segs_f, text="Do not concatenate samples", variable=segs, onvalue=True, offvalue=False, **check_args, command=lambda: twiargs.setbool('--segs', segs.get()))
  segs_check.pack(side=tk.LEFT)

  nochat   = tk.BooleanVar(False)
  nochat_y = segs_y + entry_h + sep
  nochat_f = tk.Frame(main_f, bg=main_bg)
  nochat_f.place(relx=.05, rely=nochat_y, relwidth=1, relheight=entry_h)
  nochat_check = tk.Checkbutton(nochat_f, text="Ignore chat data", variable=nochat, onvalue=True, offvalue=False, **check_args, command=lambda: twiargs.setbool('--nochat', nochat.get()))
  nochat_check.pack(side=tk.LEFT)

  signal   = tk.BooleanVar(False)
  signal_y = nochat_y + entry_h + sep
  signal_f = tk.Frame(main_f, bg=main_bg)
  signal_f.place(relx=.05, rely=signal_y, relwidth=1, relheight=entry_h)
  signal_check = tk.Checkbutton(signal_f, text='Look for !twigen in chat and include that moment', variable=signal, onvalue=True, offvalue=False, **check_args, command=lambda: twiargs.setbool('--signal', signal.get()))
  signal_check.pack(side=tk.LEFT)

  muted   = tk.BooleanVar(False)
  muted_y = signal_y + entry_h + sep
  muted_f = tk.Frame(main_f, bg=main_bg)
  muted_f.place(relx=.05, rely=muted_y, relwidth=1, relheight=entry_h)
  muted_check = tk.Checkbutton(muted_f, text='Do not ignore muted segments', variable=muted, onvalue=True, offvalue=False, **check_args, command=lambda: twiargs.setbool('--muted', muted.get()))
  muted_check.pack(side=tk.LEFT)

  tmp   = tk.BooleanVar(False)
  tmp_y = muted_y + entry_h + sep
  tmp_f = tk.Frame(main_f, bg=main_bg)
  tmp_f.place(relx=.05, rely=tmp_y, relwidth=1, relheight=entry_h)
  tmp_check = tk.Checkbutton(tmp_f, text='Use a temporary directory to save downloaded data', variable=tmp, onvalue=True, offvalue=False, **check_args, command=lambda: twiargs.setbool('--tmp', tmp.get()))
  tmp_check.pack(side=tk.LEFT)

  # ok/cancel button
  ok_y = tmp_y + entry_h*2 + sep
  ok_f = tk.Frame(main_f, bg=main_bg)
  ok_f.place(relx=.05, rely=ok_y, relwidth=1, relheight=entry_h*2)
  ok_button = tk.Button(ok_f, text="  OK  ", highlightbackground=main_bg, activebackground='#4B367C', activeforeground='#FFFFFF', **label_args, command=lambda: twiargs.runwith(('--urls', urls.get()), ('--include', incs.get()), ('--output', out.get()), ('--min', mins.get())))
  ok_button.pack(side=tk.LEFT)

  win.mainloop()
