import re
import os
import time
import tempfile
import subprocess as sp

from twigen import gen

class Proc:
  def __init__(self, vid, tmp=False):
    self.workdir(vid, tmp)

  #create the workspace of the process
  def workdir(self, vid, tmp):
    if tmp:
      self.outdir_tmp = tempfile.TemporaryDirectory(suffix="_twigen_data")
      self.tsdir_tmp  = tempfile.TemporaryDirectory(suffix="_twigen_ts")
      self.outdir     = self.outdir_tmp.name+'/'
      self.tsdir      = self.tsdir_tmp.name+'/'
    else:
      base = "twigen_data/"
      self.outdir = base+vid+'/'
      self.tsdir  = self.outdir+"ts/"
      if not os.path.exists(base):
        os.mkdir(base)
      if not os.path.exists(self.outdir):
        os.mkdir(self.outdir)
      if not os.path.exists(self.tsdir):
        os.mkdir(self.tsdir)

#return url and video id given a url or video id
def read_url(s):
  if '/' not in s:
    url = "https://www.twitch.tv/videos/"+s
    vid = s
  else:
    url = s
    vid = self.url.split('/')[-1]
  return url, vid

#return direct url of video (m3u8 index)
def link(url):
  args = ['youtube-dl','-g',url]
  r = sp.run(args, stdout=sp.PIPE, stderr=sp.DEVNULL, text=True)
  return r.stdout[:-1]
  
#return path to m3u8 index
def m3u8(outdir, link):
  path = outdir+'/index.m3u8'
  sp.run(['curl',link,'-o',path], stdout=sp.DEVNULL, stderr=sp.DEVNULL)
  return path

#return total length of video and length of each ts file
def parse_m3u8(m3u8):
  rgx = re.compile("^#EXTINF:((\d+\.?\d*)|(\.\d+))")
  tot = re.compile("TOTAL-SECS:((\d+\.?\d*)|(\.\d+))")
  total_len, ts_len = .0 , .0
  with open(m3u8) as f:
    for line in f:
      m = rgx.findall(line)
      n = tot.findall(line)
      if len(n) != 0:
        total_len = float(n[0][0])
      if len(m) != 0:
        ts_len = float(m[0][0])
        break
  return total_len, ts_len

#return list of ts file names
def parse_ts(m3u8):
  ts  = []
  rgx = re.compile("^#")
  with open(m3u8) as f:
    for line in f:
      m1 = rgx.findall(line)
      if len(m1) == 0:
        ts.append(line[:-1])
  return ts

#return link (partial url) for the ts files
def tslink(link):
  return '/'.join(link.split('/')[:-1])+'/'

#return list of ts file names related to the given samples
def uts(samples, ts, ts_len):
  res = []
  for i in range(0, len(samples)): #refactor
    b = int(gen.sl(samples[i].begin.total_seconds(), ts_len)/ts_len)
    e = int(gen.sl(samples[i].end.total_seconds(), ts_len)/ts_len)
    res.extend(ts[b:e])
  return res

#return total length of the ts files as a timedelta object
def conlen(ts, ts_len):
  return gen.tdelta(seconds=len(ts) * ts_len)

#download chat data and return path to file
def dchat(outdir, vid, quiet=False):
  args = ['tcd','-v',vid,'-o',outdir]
  if os.path.exists(outdir+'/'+vid+'.txt'):
    if not quiet:
      print("Chat data perviously downloaded.")
  elif not quiet:
    print("Downloading chat data ... ", end='', flush=True)
    sp.run(args)
  else:
    sp.run(args, stdout=sp.DEVNULL, stderr=sp.DEVNULL)
  return os.path.abspath(outdir+'/'+vid+'.txt')

# filter muted ts files
def nonmuted(ts, muted=False, quiet=False):
  if muted:
    return ts
  len_ts = len(ts)
  pred = lambda x: len(re.compile("muted").findall(x)) == 0
  ts = list(filter(pred, ts))
  if len_ts > len(ts) and not quiet:
    print("Muted segments ignored.")
  return ts

# download video
def dvod(tsdir, ts, ts_link, parse_ts=None, quiet=False, muted=False):
  # filter muted segments
  ts = nonmuted(ts, muted, quiet)
  if parse_ts:
    parse_ts()
  # check if ts files have been previously downloaded
  files   = os.listdir(tsdir)
  d, diff = [], []
  len_ts  = len(ts)
  ldir    = len(files)
  if ldir != 0:
    d    = list(set(ts).intersection(files))
    diff = list(set(ts)-set(d))
    if len(diff) != 0:
      # set ts to the files that needs to be downloaded
      ts = diff
      if not quiet and len(d) != 0:
        print(str(len(d))+" of "+str(len_ts)+" segments perviously downloaded.")
    elif not quiet:
      print("All segments perviously downloaded.")
      return ts
  # build the arguments for curl
  args = ['curl']
  for ts_file in ts:
    args.extend([ts_link+ts_file,'-o',tsdir+'/'+ts_file])
  if not quiet:
    print("Downloading "+str(len(ts))+" segments ...")
    proc = sp.Popen(args, stdout=sp.DEVNULL, stderr=sp.DEVNULL)
    # print progress by inspecting tsdir
    tsnow, notd = 0, 0
    ldir  = len(os.listdir(tsdir))
    if ldir != 0:
      if len(d) == 0:
        tsnow = ldir
      else:
        notd = ldir-len(d)
    while True:
      ldir = len(os.listdir(tsdir))
      check = ldir-len(d)-tsnow-notd
      progress = int(check/len(ts)*100)
      print(str(progress)+'%', end='\r')
      if progress == 100:
        print('100%')
        break
      if progress > 100: # shouldnt happen
        break
      time.sleep(0.5)
    proc.wait()
  else:
    sp.run(args, stdout=sp.DEVNULL, stderr=sp.DEVNULL)
  # if ts = diff, reconstruct ts with all the original ts files
  if len(d) != 0:
    ts = ts_recons(ts, d)
  return ts

# extend ts with d and sort it
def ts_recons(ts, d):
  ts.extend(d)
  ts_int = []
  rgx = re.compile("^[0-9]+")
  for ts_file in ts:
    m = rgx.findall(ts_file)
    if len(m) != 0:
      ts_int.append(int(m[0]))
  zips = list(zip(ts, ts_int))
  zips.sort(key=lambda t: t[1])
  ts = []
  for t in zips:
    ts.append(t[0])
  return ts

#concatenate ts files
def con(tsdir, ts, out, quiet=False, group=False):
  if not quiet:
    print("Editing output file "+out+" ... ", end='')
  args = ['ffmpeg','-y','-i']
  #concat first ts file
  if not group:
    arg = 'concat:'+tsdir+ts[0]
  else:
    arg = 'concat:'+ts[0]
  #concat the rest of ts files
  for tsfile in ts[1:]:
    if not group:
      arg += '|'+tsdir+tsfile
    else:
      arg += '|'+tsfile
  args.extend([arg,'-c','copy',out])
  sp.run(args, stdout=sp.DEVNULL, stderr=sp.DEVNULL)
  if not quiet:
    print("Done.")

#concatenate ts files by samples #TODO refactor
def conpart(tsdir, ts, out, sample_len, ts_len, quiet=False, group=False):
  if not quiet:
    print("Writing segments ... ", end='')
  ts_many = int(sample_len/ts_len)
  loop = (len(ts)//ts_many)-1
  args = ['ffmpeg','-y','-i']
  i = 0
  name = 0
  for _ in range(1, loop):
    #concat first ts file of sample
    if not group:
      arg = 'concat:'+tsdir+ts[i]
    else:
      arg = 'concat:'+ts[i]
    #concat ts_many files
    for j in range(i+1, i+ts_many):
      if not group:
        arg += '|'+tsdir+ts[j]
      else:
        arg += '|'+ts[j]
    o = out.split('.')
    outs = o[0]+'_'+str(name)+'.'+o[1]
    args.extend([arg,'-c','copy',outs])
    sp.run(args, stdout=sp.DEVNULL, stderr=sp.DEVNULL)
    args = ['ffmpeg','-y','-i']
    name += 1
    i += ts_many
  ts_many = (len(ts)//ts_many)+len(ts)%ts_many
  while True:
    #concat first ts file of sample
    if not group:
      arg = 'concat:'+tsdir+ts[i]
    else:
      arg = 'concat:'+ts[i]
    #concat ts_many files
    for j in range(i+1, i+ts_many):
      if j >= len(ts):
        break
      if not group:
        arg += '|'+tsdir+ts[j]
      else:
        arg += '|'+ts[j]
    o = out.split('.')
    outs = o[0]+'_'+str(name)+'.'+o[1]
    args.extend([arg,'-c','copy',outs])
    sp.run(args, stdout=sp.DEVNULL, stderr=sp.DEVNULL)
    args = ['ffmpeg','-y','-i']
    name += 1
    i += ts_many
    if i >= len(ts):
      break
  if not quiet:
    print("Done.")
